@@ -126,7 +126,7 @@ mysqlx_property_get_value(zval * object, zval * member, int type, void ** cache_
 
 
 /* {{{ mysqlx_property_set_value */
-void
+property_set_value_return_type
 mysqlx_property_set_value(zval * object, zval * member, zval * value, void **cache_slot)
 {
 	zval tmp_member;
@@ -158,7 +158,12 @@ mysqlx_property_set_value(zval * object, zval * member, zval * value, void **cac
 	if (member == &tmp_member) {
 		zval_dtor(member);
 	}
+
+	#if PHP_VERSION_ID >= 70400 // PHP 7.4 or newer
+	DBG_RETURN(value);
+	#else // PHP older than 7.4
 	DBG_VOID_RETURN;
+	#endif
 }
 /* }}} */
 
