@@ -41,11 +41,16 @@ struct st_mysqlx_property
 	func_mysqlx_property_set set_value;
 };
 
+#if PHP_VERSION_ID >= 70400 // PHP 7.4 or newer
+using property_set_value_return_type = zval*;
+#else // PHP older than 7.4
+using property_set_value_return_type = void;
+#endif
 
 void mysqlx_add_properties(HashTable * ht, const st_mysqlx_property_entry* entries);
 
 zval * mysqlx_property_get_value(zval * object, zval * member, int type, void ** cache_slot, zval * rv);
-void mysqlx_property_set_value(zval * object, zval * member, zval * value, void ** cache_slot);
+property_set_value_return_type mysqlx_property_set_value(zval * object, zval * member, zval * value, void ** cache_slot);
 int mysqlx_object_has_property(zval * object, zval *member, int has_set_exists, void ** cache_slot);
 
 void mysqlx_free_property_cb(zval *el);
