@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2018 The PHP Group                                |
+  | Copyright (c) 2006-2019 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -202,6 +202,7 @@ typedef std::shared_ptr< xmysqlnd_session > XMYSQLND_SESSION;
 typedef std::shared_ptr<xmysqlnd_session_data> XMYSQLND_SESSION_DATA;
 
 using vec_of_addresses = util::vector< std::pair<util::string,long> >;
+using vec_of_attribs = util::vector< std::pair<util::string,util::string > >;
 
 /* {{{ list_of_addresses_parser */
 class list_of_addresses_parser
@@ -438,12 +439,14 @@ public:
 	const MYSQLND_ERROR_INFO* get_error_info() const;
 
 	enum_func_status  set_client_option(enum_xmysqlnd_client_option option, const char * const value);
+	enum_func_status  send_client_attributes();
 
-	enum_func_status  send_reset();
+	enum_func_status  send_reset(bool keep_open);
 	enum_func_status  send_close();
 	bool is_closed() const { return state.get() == SESSION_CLOSED; }
 	size_t            negotiate_client_api_capabilities(const size_t flags);
 
+	bool is_session_properly_supported() const;
 	size_t            get_client_id();
 	void              cleanup();
 public:
@@ -451,14 +454,14 @@ public:
 	XMYSQLND_L3_IO	                   io;
 	/* Authentication info */
 	std::unique_ptr<Session_auth_data> auth;
-	Auth_mechanisms auth_mechanisms;
+	Auth_mechanisms                    auth_mechanisms;
 	/* Other connection info */
 	std::string scheme;
 	std::string default_schema;
 	transport_types                    transport_type;
 	/* Used only in case of non network transports */
-	std::string socket_path;
-	std::string server_host_info;
+	std::string                        socket_path;
+	std::string                        server_host_info;
 	size_t			                   client_id;
 	const MYSQLND_CHARSET*             charset;
 	/* If error packet, we use these */
@@ -467,6 +470,13 @@ public:
 	/* Operation related */
 	XMYSQLND_SESSION_STATE             state;
 	size_t			                   client_api_capabilities;
+	/*
+		before 8.0.16 session wasn't properly supported:
+		- it wasn't possible to reset session without reauthentication
+		- Session::Close meant Connection::Close
+		more details: WL#12375 WL#12396
+	*/
+	mutable boost::optional<bool> session_properly_supported;
 	/* stats */
 	MYSQLND_STATS*                     stats;
 	zend_bool		                   own_stats;
@@ -474,8 +484,11 @@ public:
 	zend_bool persistent{ TRUE };
 	/* Seed for the next transaction savepoint identifier */
 	unsigned int                       savepoint_name_seed;
+	vec_of_attribs                     connection_attribs;
+	drv::Prepare_stmt_data             ps_data;
 private:
 	void free_contents();
+	Mysqlx::Datatypes::Object*  prepare_client_attr_object();
 };
 
 
@@ -645,7 +658,6 @@ public:
 
 	xmysqlnd_schema* create_schema_object(const MYSQLND_CSTRING schema_name);
 
-
     const enum_func_status close(const enum_xmysqlnd_session_close_type close_type);
 	bool is_closed() const { return data->is_closed(); }
 
