@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2018 The PHP Group                                |
+  | Copyright (c) 2006-2019 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -18,6 +18,7 @@
 #include "php_api.h"
 #include "mysqlnd_api.h"
 #include "xmysqlnd.h"
+#include "xmysqlnd_zval2any.h"
 
 #include "util/string_utils.h"
 
@@ -30,6 +31,27 @@ namespace drv {
 
 using namespace Mysqlx::Datatypes;
 
+namespace {
+
+void zval2object(zval* zv, Mysqlx::Datatypes::Any& any)
+{
+	any.set_type(Any_Type_OBJECT);
+	Mysqlx::Datatypes::Object* obj{ any.mutable_obj() };
+	HashTable* properties{ zend_std_get_properties(zv) };
+	zend_string* property_name{nullptr};
+	zval* property_value{nullptr};
+	MYSQLX_HASH_FOREACH_STR_KEY_VAL(properties, property_name, property_value) {
+		if (property_name && property_value) {
+			Mysqlx::Datatypes::Object_ObjectField* field{ obj->add_fld() };
+			field->set_key(ZSTR_VAL(property_name), ZSTR_LEN(property_name));
+			Mysqlx::Datatypes::Any* field_value{ field->mutable_value() };
+			zval2any(property_value, *field_value);
+		}
+	} ZEND_HASH_FOREACH_END();
+}
+
+} // anonymous namespace
+
 /* {{{ zval2any */
 PHP_MYSQL_XDEVAPI_API enum_func_status
 zval2any(const zval * const zv, Mysqlx::Datatypes::Any & any)
@@ -90,7 +112,7 @@ zval2any(const zval * const zv, Mysqlx::Datatypes::Any & any)
 		}
 		case IS_OBJECT: {
 			DBG_INF("IS_OBJECT");
-			any.set_type(Any_Type_OBJECT);
+			zval2object(const_cast<zval*>(zv), any);
 			break;
 		}
 		default:
@@ -228,26 +250,24 @@ any2zval(const Mysqlx::Datatypes::Any & any, zval * zv)
 		case Any_Type_OBJECT: {
 			zval properties;
 			ZVAL_UNDEF(&properties);
-			array_init_size(&properties, any.obj().fld_size());
+			const int fields_count{ any.obj().fld_size() };
+			array_init_size(&properties, fields_count);
 
-			object_init(zv);
-			for (int i{0}; i < any.obj().fld_size(); ++i) {
+			for (int i{0}; i < fields_count; ++i) {
 				zval entry;
 				ZVAL_UNDEF(&entry);
-				any2zval(any.obj().fld(i).value(), &entry);
+				const auto& field{ any.obj().fld(i) };
+				any2zval(field.value(), &entry);
 				if (Z_REFCOUNTED(entry)) {
 					Z_ADDREF(entry);
 				}
-				add_assoc_zval_ex(zv, any.obj().fld(i).key().c_str(), any.obj().fld(i).key().size(), &entry);
-				zend_hash_next_index_insert(Z_ARRVAL_P(zv), &entry);
-				zval_ptr_dtor(zv);
+				add_assoc_zval_ex(&properties, field.key().c_str(), field.key().size(), &entry);
+				zend_hash_next_index_insert(Z_ARRVAL(properties), &entry);
 			}
 
-			if (!zend_standard_class_def->default_properties_count && !zend_standard_class_def->__set) {
-				Z_OBJ_P(zv)->properties = Z_ARR(properties);
-			} else {
-				zend_merge_properties(zv, Z_ARRVAL(properties));
-			}
+			object_init(zv);
+			zend_merge_properties(zv, Z_ARRVAL(properties));
+			zval_ptr_dtor(&properties);
 			break;
 		}
 		case Any_Type_ARRAY:
